/*
	Copyright (C) 2012-2030

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "ConfigFile.h"
#include "StringUtils.h"
#include <cstring>

using namespace Config;
using namespace std;
using namespace Utilities::Strings;



const char* File::c_sDelimiter = "\n";
const char* File::c_sAssigner = "=";
const char* File::c_sVarPrefix = "[";
const char* File::c_sVarPostfix = "]";


File::File ()
{
	//Clear ();
}


void File::Clear ()
{
	memset ( this, 0, sizeof( Config::File ) );
	//cout << "\nClearing: Config file size=" << strlen( cData );
	//cout << "\nClearing: size of Config::File=" << sizeof( Config::File );
}


bool File::Load ( string NameOfFile )
{
	int size, result;
	ifstream *fFile;
	//FILE* fFile;
	
	//fFile = fopen ( NameOfFile.c_str(), "rb" );
	
	//if ( !fFile ) return false;
	
	fFile = new ifstream( NameOfFile.c_str()/*, ios::binary*/ );
	//result = fread ( cData, 1, size /*c_iConfigFile_MaxSize*/, fFile );
	
	if ( !fFile->is_open() || fFile->fail() ) return false;
	
	//fseek( fFile, 0, SEEK_END );
	//size = ftell( fFile );
	//fseek( fFile, 0, SEEK_SET );
	fFile->seekg ( 0, std::ifstream::end );
	size = fFile->tellg ();
	fFile->seekg ( 0, std::ifstream::beg );
	
	
	if ( !fFile->is_open() || fFile->fail() ) return false;
	//if ( !result ) return false;
	
	fFile->read ( cData, size );
	result = fFile->gcount();
	
	// terminate string
	cData [ result ] = 0;
	
	// close the file when done
	fFile->close ();
	//fclose ( fFile );
	
	cout << "\nLoading: Config file size=" << strlen( cData );
	
	delete fFile;
	
	return true;
}



bool File::Save ( string NameOfFile )
{
	ofstream *fFile;
	fFile = new ofstream( NameOfFile.c_str(), ios::trunc /*ios::binary*/ );
	
	if ( !fFile->is_open() || fFile->fail() ) return false;
	
	//fFile->write ( sData.c_str(), sData.size() );
	fFile->write ( cData, strlen(cData) );
	
	// close the file when done writing to it
	fFile->close ();
	
	cout << "\nSaving: Config file size=" << strlen( cData );
	
	delete fFile;
	
	return true;
}



bool File::Get_Value32 ( string VarName, long& Value )
{
	int x0, x1;
	string sVar;
	
	VarName = c_sVarPrefix + VarName + c_sVarPostfix;
	
	// find where variable is in string
	x0 = InStr ( cData, VarName );
	
	// if it is not there, then return false
	if ( x0 == string::npos ) return false;
	
	// look for assignment operator
	x0++;
	x0 = InStr ( cData, c_sAssigner, x0 );
	
	// look for delimiter
	x0++;
	x1 = InStr ( cData, c_sDelimiter, x0 );
	
	// get the string
	sVar = Mid ( cData, x0, x1 - x0 );
	
	if ( !isNumeric ( sVar ) ) return false;
	
	// convert to number
	Value = CLng ( sVar );
	
	// successful
	return true;
}

bool File::Get_Value64 ( string VarName, long long& Value )
{
	int x0, x1;
	string sVar;
	
	VarName = c_sVarPrefix + VarName + c_sVarPostfix;
	
	// find where variable is in string
	x0 = InStr ( cData, VarName );
	
	// if it is not there, then return false
	if ( x0 == string::npos ) return false;
	
	// look for assignment operator
	x0++;
	x0 = InStr ( cData, c_sAssigner, x0 );
	
	// look for delimiter
	x0++;
	x1 = InStr ( cData, c_sDelimiter, x0 );
	
	// get the string
	sVar = Mid ( cData, x0, x1 - x0 );
	
	if ( !isNumeric ( sVar ) ) return false;
	
	// convert to number
	Value = CLngLng ( sVar );
	
	// successful
	return true;
}


bool File::Get_String ( string VarName, string& Value )
{
	int x0, x1;
	string sVar;
	
	VarName = c_sVarPrefix + VarName + c_sVarPostfix;
	
	// find where variable is in string
	x0 = InStr ( cData, VarName );
	
	// if it is not there, then return false
	if ( x0 == string::npos ) return false;
	
	// look for assignment operator
	x0++;
	x0 = InStr ( cData, c_sAssigner, x0 );
	
	// look for delimiter
	x0++;
	x1 = InStr ( cData, c_sDelimiter, x0 );
	
	// get the string
	sVar = Mid ( cData, x0, x1 - x0 );
	
	//if ( !isNumeric ( sVar ) ) return false;
	
	// convert to string
	Value = sVar;
	
	// successful
	return true;
}


void File::Set_Value32 ( string VarName, long Value )
{
	VarName = c_sVarPrefix + VarName + c_sVarPostfix + c_sAssigner + CStr ( Value ) + c_sDelimiter;
	strcat ( cData, VarName.c_str () );
}

void File::Set_Value64 ( string VarName, long long Value )
{
	VarName = c_sVarPrefix + VarName + c_sVarPostfix + c_sAssigner + CStr ( Value ) + c_sDelimiter;
	strcat ( cData, VarName.c_str () );
}

void File::Set_String ( string VarName, string Value )
{
	VarName = c_sVarPrefix + VarName + c_sVarPostfix + c_sAssigner + Value + c_sDelimiter;
	strcat ( cData, VarName.c_str () );
}






